package be.iminds.ilabt.jfed.highlevel.jobs.sfa_parts;

import be.iminds.ilabt.jfed.experiment.SfaExperimentPart;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.highlevel.DesiredStatus;
import be.iminds.ilabt.jfed.highlevel.jobs.Job;
import be.iminds.ilabt.jfed.highlevel.jobs.UpdateUntilReadyStateSlice;
import be.iminds.ilabt.jfed.highlevel.jobs.report.JobReportWrappingLogger;
import be.iminds.ilabt.jfed.highlevel.jobs.states.JobStateFactory;
import be.iminds.ilabt.jfed.highlevel.jobs.states.UpdateUntilFinalOpStateTaskState;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Duration;

/**
 * User: twalcari
 * Date: 1/14/2016
 * Time: 12:05
 */
public class UpdateSfaExperimentPartUntilTargetOpStatusSlice extends UpdateUntilReadyStateSlice<SfaExperimentPart> {
    private static final Logger ACTUAL_LOG = LoggerFactory.getLogger(UpdateSfaExperimentPartUntilTargetOpStatusSlice.class);
    private final Logger LOG;

    @Nonnull private final JobStateFactory jobStateFactory;
    @Nonnull private final DesiredStatus desiredStatus;
    private boolean timeout;

    public UpdateSfaExperimentPartUntilTargetOpStatusSlice(@Nonnull Job<?> job,
                                                           @Nonnull SfaExperimentPart experimentPart,
                                                           @Nullable Duration checkReadyInterval,
                                                           @Nullable Duration waitForReadyDelay,
                                                           @Nonnull JobStateFactory jobStateFactory,
                                                           @Nonnull DesiredStatus desiredStatus) {
        super(job, experimentPart, checkReadyInterval, waitForReadyDelay);
        this.jobStateFactory = jobStateFactory;
        this.desiredStatus = desiredStatus;
        this.timeout = false;
        this.LOG = new JobReportWrappingLogger(ACTUAL_LOG, job.getJobReport());
    }

    @Override
    public ExperimentTaskStatus statefulRun() throws JFedException, InterruptedException {
        UpdateUntilFinalOpStateTaskState statusState =
                jobStateFactory.createGetStatusUntilReadyOrFailTaskState(job, LOG,
                        checkReadyInterval, waitForReadyDelay, experimentPart,
                        desiredStatus);

        statusState.setCheckReadyInterval(checkReadyInterval);

        setAndRunState(statusState);
        this.timeout = statusState.isFailedDueToTimeout();
        if (timeout)
            LOG.debug("UpdateSfaExperimentPartUntilTargetOpStatusSlice detected timeout");
        return statusState.getStatus();
    }

    public boolean isTimeout() {
        return timeout;
    }
}
